@@ -4,7 +4,7 @@ Utilities for making database connections easier.
 
 import MySQLdb
 
-from main import settings
+import settings
 
 
 def run_query(database, query):
