@@ -2,6 +2,8 @@ import datetime
 import warnings
 
 from connection import DB
+from group_by import GroupBy
+from join import TableBuilder
 from table import Table
 from main import get_class
 
@@ -11,11 +13,11 @@ class Fact(Table):
     Fact base class.
     
     """
-    dim_names = []
-    dim_fields = []
-    dim_modules = []
     dim_classes = []
+    dim_fields = []
     dim_map = {}
+    dim_modules = []
+    dim_names = []
     historical_iterations = 100
     
     def __init__(self, *args, **kwargs):
@@ -58,18 +60,18 @@ class Fact(Table):
         gives the mapping for all the dimensions linked to the fact.
         Sets self.dim_classes to a list of classes - one for each dimension.
         
-        Example usage :
+        Example usage:
         > _import_dimensions()
         Example of self.dim_map :
         > {
-        >     'location' : {
-        >         'LON':1,
-        >         'NY':2,
+        >     'location': {
+        >         'LON': 1,
+        >         'NY': 2,
         >     },
-        >     'thingtocount' : {
-        >         '123':1,
-        >         'ABC88':2,
-        >         'XXX11':3,
+        >     'thingtocount': {
+        >         '123': 1,
+        >         'ABC88': 2,
+        >         'XXX11': 3,
         >         ...
         >     }
         >     ...
@@ -77,11 +79,9 @@ class Fact(Table):
 
         Example of self.dim_classes:
         > [pointer to location class, pointer to home class]
-        > self.dim_classes[1].get_dictionnary('short_code')
+        > self.dim_classes[1].get_dictionary('short_code')
 
         """
-        # First thing to understand is why dim_names and dim_fields are 
-        # separate...
         for dim_name, dim_field in zip(self.dim_names, self.dim_fields):
             
             if dim_field == None:
@@ -110,10 +110,7 @@ class Fact(Table):
         """
         result = []
         error = False
-
-        if self.dim_map == {}:
-            self._import_dimensions()
-        
+                
         for (value, dim_name) in zip(src_tuple, self.dim_names):
             if self.dim_map[dim_name] == None:
                 result.append(value)
@@ -141,22 +138,7 @@ class Fact(Table):
                 dimension.update()
         super(Fact, self).build()
     
-    def update(self, historical=False, index=0):
-        """
-        Updates the fact table with the newest rows (modified since last
-        update).
-        """
-        # Make sure all the tables have been created:
-        self.build()
-        
-        # Status.
-        msg = "Updating %s" % self.table_name
-        self._print_status(msg)
-
-        error_count = 0
-        success_count = 0
-        
-        # Get the query.
+    def _get_query(self, historical, index):
         if not historical:
             query = self.source_query
         else:
@@ -165,13 +147,14 @@ class Fact(Table):
                 return 0
             else:
                 query = self.historical_source_query.format(index)
+        return query
+    
+    def _insert_rows(self, data):
+        error_count = 0
+        success_count = 0
+        
+        self._import_dimensions()
         
-        # Get the full source list.
-        data = []
-        with DB(self.source_db) as database:
-            data = database.execute(query)
-
-        # Update the fact table with all the rows.
         for row in data:
             map_result = self._map_tuple(self._transform_tuple(row))
             destination_tuple = map_result[0]
@@ -186,24 +169,78 @@ class Fact(Table):
                                                     len(destination_tuple)))
                     self.connection.execute(query, destination_tuple)
                     success_count += 1
-                except Exception as e:
-                    print "--> MySQL error: %s" % str(destination_tuple)
-                    print "- Row after _transform_tuple(): %s" % str(
-                                                self._transform_tuple(row))
-                    print "- Raw row from DB: %s" % str(row)
-                    print(e)
+                except Exception, e:
+                    self._print_status("MySQL error: %s" % str(
+                                                            destination_tuple))
+                    self._print_status("Row after _transform_tuple(): %s" % (
+                                            str(self._transform_tuple(row))))
+                    self._print_status("Raw row from DB: %s" % str(row))
+                    self._print_status(e)
                     error_count += 1
             else:
-                print "--> Error on mapping: %s" % str(destination_tuple)
-                print "- Row after _transform_tuple(): %s" % str(
+                print "Error on mapping: %s" % str(destination_tuple)
+                print "Row after _transform_tuple(): %s" % str(
                                                 self._transform_tuple(row))
-                print "- Raw row from DB: %s" % str(row)
+                print "Raw row from DB: %s" % str(row)
                 error_count += 1
         
-        msg = "--> %s rows inserted, %s errors (i.e. rows not inserted)" % (
+        msg = "%s rows inserted, %s errors (i.e. rows not inserted)" % (
                                                     success_count, error_count)
         self._print_status(msg)
     
+    def join_query(self, historical, index):
+        table_builder = TableBuilder(
+            main_db=self.source_db,
+            main_query=self.source_query,
+            create_query=None,
+            output_table=self.table_name,
+            verbose=True
+            )
+        for extra_query in self.extra_queries:
+            query_dict = getattr(self, extra_query)
+            table_builder.add_source(
+                extra_query,
+                query_dict['db'],
+                query_dict['query'],
+                query_dict['join_on'],
+                query_dict['outer_join'],
+                )
+        table_builder.join()
+        if hasattr(self, 'group_by'):
+            group_by = self.group_by
+            group_by = GroupBy(table_builder.result, group_by)
+            self._insert_rows(group_by.process())
+        else:
+            self._insert_rows(table_builder.result)
+
+    def single_query(self, historical, index):
+        # Get the query.
+        query = self._get_query(historical, index)
+
+        # Execute the select query.
+        data = []
+        with DB(self.source_db) as database:
+            data = database.execute(query)
+        
+        # Update the fact table with all the rows.
+        self._insert_rows(data)
+
+    def update(self, historical=False, index=0):
+        """
+        Updates the fact table with the newest rows.
+        """
+        # Make sure all the tables have been created.
+        self.build()
+        
+        # Status.
+        self._print_status("Updating %s" % self.table_name)
+        
+        # If join is required, need to do things differently.
+        if hasattr(self, 'extra_queries') and self.extra_queries:
+            self.join_query(historical, index)
+        else:
+            self.single_query(historical, index)
+
     def historical(self):
         """
         Run the historical_query - useful for rebuilding the tables from
